<?php
namespace LavenderBits\Squaby;

use Kirby\Cms\Page;
use Kirby\Http\Response;

class SignupController
{
    public static function routes(): array
    {
        return [
            [
                "pattern" => "/newsletter-api/signup",
                "method" => "POST",
                "action" => function () {
                    return SignupController::signup();
                },
            ],
            [
                "pattern" => "/newsletter-api/confirm/(:any)/(:any)",
                "method" => "GET",
                "action" => function ($newsletterId, $secret) {
                    return SignupController::confirmSubscription(
                        $newsletterId,
                        $secret
                    );
                },
            ],
            [
                "pattern" => "/newsletter-api/unsubscribe/(:any)/(:any)",
                "method" => "GET|POST",
                "action" => function ($newsletterId, $secret) {
                    if (kirby()->request()->method() === "GET") {
                        return SignupController::unsubscribePage(
                            $newsletterId,
                            $secret
                        );
                    } else {
                        return SignupController::unsubscribe(
                            $newsletterId,
                            $secret
                        );
                    }
                },
            ],
        ];
    }

    public static function signup(): Response
    {
        // TODO: Spam prevention and e-mail validation, actually
        /** @var Newsletter */
        $newsletter = page(get("newsletter"));
        $success = $newsletter->addSubscriber(get("email"));
        return Response::json(["success" => $success]);
    }

    public static function confirmSubscription(
        string $newsletterId,
        string $secret
    ): Response|false {
        /** @var Newsletter */
        $newsletter = page($newsletterId);
        if (!$newsletter) {
            return false;
        }

        /** @var Subscriber */
        $subscription = $newsletter
            ->childrenAndDrafts()
            ->findBy("secret", $secret);
        if (!$subscription) {
            return false;
        }

        $subscription = $subscription->confirm();
        return Response::go($newsletter->confirmPage()->toPage()->url());
    }

    public static function unsubscribePage(
        string $newsletterId,
        string $secret
    ): Page|false {
        /** @var Newsletter */
        $newsletter = page($newsletterId);
        if (!$newsletter) {
            return false;
        }

        /** @var Subscriber */
        $subscription = $newsletter
            ->childrenAndDrafts()
            ->findBy("secret", $secret);
        if (!$subscription) {
            return false;
        }

        //TODO: Make all of this variable and editable somehow
        return Page::factory([
            "template" => "squaby-unsubscribe",
            "slug" =>
                "newsletter-api/unsubscribe/" . $newsletterId . "/" . $secret,
            "content" => [
                "title" => "Unsubscribe",
            ],
        ]);
    }

    public static function unsubscribe(
        string $newsletterId,
        string $secret
    ): Response|false {
        /** @var Newsletter */
        $newsletter = page($newsletterId);
        if (!$newsletter) {
            return false;
        }

        /** @var Subscriber */
        $subscription = $newsletter
            ->childrenAndDrafts()
            ->findBy("secret", $secret);
        if (!$subscription) {
            return false;
        }

        $subscription = $subscription->cancel();
        return Response::go($newsletter->unsubscribePage()->toPage()->url());
    }
}
